import { DefaultInputReader } from "./defaultinput";
import { NetplayPlayer } from "./types";
import { GameClass } from "./game";
import Peer from "peerjs";
export declare abstract class GameWrapper {
    gameClass: GameClass;
    /** The canvas that the game will be rendered onto. */
    canvas: HTMLCanvasElement;
    /** The network stats UI. */
    stats: HTMLDivElement;
    /** The floating menu used to select a match. */
    menu: HTMLDivElement;
    inputReader: DefaultInputReader;
    stateSyncPeriod: number;
    isChannelOrdered(channel: RTCDataChannel): boolean;
    isChannelReliable(channel: RTCDataChannel): boolean;
    checkChannel(channel: RTCDataChannel): void;
    constructor(gameClass: GameClass);
    /**
     * Calculate a scaling for our canvas so that it fits the whole screen.
     * Center the canvas with an offset.
     */
    calculateLayout(container: {
        width: number;
        height: number;
    }, canvas: {
        width: number;
        height: number;
    }): {
        width: number;
        height: number;
        left: number;
        top: number;
    };
    /**
     * Recalculate canvas scaling / offset.
     */
    resize(): void;
    peer?: Peer;
    start(): void;
    formatRTCStats(stats: RTCStatsReport): string;
    rtcStats: string;
    watchRTCStats(connection: RTCPeerConnection): void;
    abstract startHost(players: Array<NetplayPlayer>, conn: Peer.DataConnection): any;
    abstract startClient(players: Array<NetplayPlayer>, conn: Peer.DataConnection): any;
}
